require 'ostruct'
require 'cgi'
require 'git'
require 'socket_client'
require 'eezee_api'

module E4z
  class DevBot
    def initialize(path)
      @git = Git.init(path)
      @working_dir = path
      @eezee_token = ENV['E4Z_TOKEN']
      @eezee_client = ::E4z::ApiClient.new(@eezee_token)
    end

    def run
      ::E4z::SocketClient.new(project_slug) do |data|
        instruction_id = JSON.parse(data)['instruction_id']
        instruction = @eezee_client.get_instruction(project_id, instruction_id)
        data = instruction['data']
        execute = instruction['execute']

        puts "Running `#{execute}` with #{data}"

        begin
          OpenStruct.new(data).instance_eval(execute)
        rescue Exception => e
          puts "Err log data: #{data}"
          puts "Err log execute: #{execute}"
          puts e.message
        end

        payload = { did_run: true }
        @eezee_client.update_instruction(project_id, instruction_id, payload)
      end
    end

    def project_slug
      "project-#{project_id}"
    end

    def github_repo
      @github_repo ||= @git.remotes.find { |r| r.name == 'origin' }.url.match(/:(.+)\.git/)[1]
    end

    def project_id
      @project_id ||= @eezee_client.find_project_by_github_slug(github_repo)['id']
    end
  end
end